import { Layer } from "./network";
export interface BaseOptions {
    alpha: number;
    gamma: number;
    epsilon: number;
    epsilonDecay: number;
}
interface ModelOptions extends BaseOptions {
    maxMemory: number;
    batchSize: number;
    episodeSize: number;
}
export interface CompileOptions {
    loss: "meanSquaredError" | "meanAbsoluteError";
}
export default class WASML {
    private init;
    private states;
    private actions;
    private options;
    private config;
    private layers;
    private mode;
    private DQN;
    private Target;
    private Memory;
    private QTable;
    private KARPATHY;
    private last_state;
    private last_action;
    private episode;
    private epsilon;
    /**
     * Loads the WASM binary into the browser.
     * @returns {Promise<void>} - A promise that resolves when the WASM binary has been loaded.
     */
    static load(): Promise<void>;
    /**
     * Generates a model with the given options.
     * @param {number} states - The size of the input vector space for the model.
     * @param {number} actions - The number of actions the model can take.
     * @param {ModelOptions} options - The options for the model.
     */
    model(states: number, actions: number, options?: Partial<ModelOptions>): Promise<void>;
    /**
     * Uses Andrej Karpathy's method to train the model.
     * @note For some reason, the `.model()` method trains significantly slower than this method, so keeping it for now.
     * @param {number} states - The size of the input vector space for the model.
     * @param {number} actions - The number of actions the model can take.
     * @param {ModelOptions} options - The options for the model.
     * @deprecated This method will be removed in a future release.
     */
    karparthy(states: number, actions: number, options?: Partial<ModelOptions>): Promise<void>;
    /**
     * Generates a Q-Learning table with the given options.
     * @param {number} states - The size of the input vector space for the table.
     * @param {number} actions - The number of actions the table can take.
     * @param {BaseOptions} options - The options for the table.
     */
    table(states: number, actions: number, options?: Partial<BaseOptions>): Promise<void>;
    /**
     * Checks if the model has been initialised.
     * @returns {boolean} - Whether the model has been initialised.
     */
    private initialised;
    /**
     * Adds the specified layers to the neural network.
     * @param {Layer[]} layers - The array of layers to add to the neural network.
     */
    addLayers(layers: Layer[]): void;
    /**
     * Compiles the specified model with the given options.
     * @param {Partial<CompileOptions>} options - The compilation options for the model.
     */
    compile(options?: Partial<CompileOptions>): void;
    /**
     * Trains the model given the input and output data.
     * @param {number[][]} inputs - The inputs to train the model with.
     * @param {number[][]} outputs - The outputs to train the model with.
     */
    train(inputs: number[][], outputs: number[][]): void;
    /**
     * Predicts the optimal output given the specified input.
     * @param {number[]} input - The state to predict result for.
     */
    predict(input: number[]): number;
    /**
     * Rewards the model for the previous action.
     * @param {number} reward - The reward value to give the model.
     * @param {number[]} state - The next state of the model.
     */
    reward(reward: number, state: number[]): void;
    /**
     * Imports model weights from an previously exported source.
     * @param {string} data - The JSON data to import in string form.
     */
    import(data: string): void;
    /**
     * Exports the model weights to a JSON string.
     * @returns {string} - The exported model weights and other information.
     */
    export(): string;
}
export {};
