import { CompileOptions } from ".";
import { Layer } from "./network";
export declare class Tensor {
    data: number[];
    shape: [number, number];
    /**
     * Creates a new tensor with the specified shape.
     * @param {[number, number]} shape - The rows and columns of the tensor.
     * @param {number[]} data - The data for the tensor.
     */
    constructor(shape: [number, number], data: number[]);
    /**
     * Explicitly sets the data for the tensor.
     * @param {number[]} data - The data to use for the tensor.
     */
    set(data: number[]): void;
    /**
     * Retuns the index of the maximum value in an array.
     * @param {number[]} arr - The array to find the maximum value in.
     * @returns {number} - The index of the maximum value in the tensor.
     */
    static argmax(arr: number[]): number;
    /**
     * Gets the data value at the specified row and column.
     * @param i - The row to get.
     * @param j - The column to get.
     * @returns {number} - The value at the specified row and column.
     */
    private at;
    /**
     * Checks that the current tensor shape is equal to the specified tensor.
     * @param {Tensor} t - The tensor to check the shape against.
     * @returns {boolean} - Whether or not the matrices are equal.
     */
    private compare;
    /**
     * Adds the specified tensor or number to the current tensor.
     * @param {Tensor | number} other - The tensor or number to add to the current tensor.
     * @returns {Tensor} - The resulting tensor.
     */
    add(other: Tensor | number): Tensor;
    /**
     * Subtracts the specified tensor to the current tensor.
     * @param {Tensor} other - The tensor to subtract from the current tensor.
     * @returns {Tensor} - The resulting tensor.
     */
    subtract(other: Tensor | number): Tensor;
    /**
     * Multiplies the current tensor by the specified tensor.
     * @param {Tensor} other - The tensor to multiply the current tensor by.
     * @returns {Tensor} - The resulting tensor.
     */
    multiply(other: Tensor): Tensor;
    /**
     * Calculates the element-wise multiplication of the current tensor and the specified tensor.
     * @param {Tensor | number} other - The tensor or number to multiply the current tensor by.
     * @returns {Tensor} - The resulting tensor.
     */
    dot(other: Tensor | number): Tensor;
    /**
     * Transposes the current tensor.
     * @returns {Tensor} - The transposed tensor.
     */
    transpose(): Tensor;
    /**
     * Calculates the sum of the current tensor.
     * @returns {number} - The sum of all the values in the tensor.
     */
    sum(): number;
    /**
     * Maps the current tensor to the specified function.
     * @param {Function} fn - The function to map the tensor to.
     * @returns {Tensor} - The resulting tensor.
     */
    map(fn: (x: number, i: number) => number): Tensor;
}
/**
 * Common activation functions.
 */
export declare const Activation: Record<Layer["activation"], {
    fn: (x: Tensor) => Tensor;
    derivative: (x: Tensor) => Tensor;
}>;
/**
 * Common loss functions.
 */
export declare const Loss: Record<CompileOptions["loss"], {
    fn: (x: Tensor, y: Tensor) => number;
    derivative: (x: Tensor, y: Tensor) => Tensor;
}>;
