import { CompileOptions } from "./index";
import { Tensor } from "./math";
export interface Layer {
    units: number;
    activation: "sigmoid" | "relu" | "linear" | "tanh";
}
export interface LayerInternal extends Layer {
    weights: Tensor;
}
export declare class NeuralNetwork {
    private layers;
    private options;
    private actions;
    private states;
    private alpha;
    private cache;
    /**
     * Build the neural network with the specified options.
     * @constructor
     * @param {number} states The number of input states.
     * @param {number} actions The number of output actions.
     * @param {Layer[]} layers The hidden layers to give the neural network.
     */
    constructor(states: number, actions: number, layers: Layer[], options: CompileOptions, alpha: number);
    /**
     * Returns the weights of specified layer in the network.
     * @param {number} idx The index of the layer to get the weights of.
     * @returns The weights of the specified layer.
     */
    weights(idx: number): Tensor;
    /**
     * Performs the activation function for the given input.
     * @param {Tensor} input The input tensor to perform the activation function on.
     * @param {Layer["activation"]} type The type of activation to use. (sigmoid, relu, ...)
     * @returns {Tensor} Result of the activation.
     */
    private activate;
    /**
     * Calculates the forward pass out of specified state.
     * @param {number[]} input The current state to give the network.
     * @returns {Tensor} The result for the given input.
     */
    forward(input: number[]): number[];
    /**
     * Back-propagates the loss and updates weights.
     * @param {number[]} target - The target state of the network.
     * @param {number} action - Specific index of the action to update.
     * @param {number} multiplier - An optional multiplier to apply to the loss.
     */
    backward(target: number[] | number, action?: number, multiplier?: number): void;
    load(weights: number[][]): void;
    /**
     * Exports the current network weights.
     * @returns {number[][]} - The weights of each layer.
     */
    save(): number[][];
}
